@@ -1,6 +1,7 @@
 # encoding: utf-8
 
 import django
+import sys
 import types
 
 from django.core.serializers.python import Serializer as _PythonSerializer
@@ -8,10 +9,18 @@ from django.core.serializers.xml_serializer import Serializer as _XmlSerializer
 from django.db import models
 from django.db.models.base import ModelBase
 from django.db.models.fields import Field
-from django.utils.datastructures import SortedDict
+from django.db.models.query_utils import DeferredAttribute, deferred_class_factory
 from django.utils.encoding import smart_text
+from django.utils.six import with_metaclass
+
+if sys.version_info < (2, 7):
+    # OrderedDict is new in 2.7, but django < 1.7 provides this similar thing
+    from django.utils.datastructures import SortedDict as OrderedDict
+else:
+    from collections import OrderedDict
+
+
 
-from .compat import with_metaclass
 
 class TypedModelManager(models.Manager):
     def get_queryset(self):
@@ -68,7 +77,7 @@ class TypedModelMetaclass(ModelBase):
             base_class = None
 
         if base_class:
-            if not hasattr(base_class, 'original'):
+            if not hasattr(base_class._meta, 'original'):
                 class original_meta:
                     proxy = True
                 Original = super(TypedModelMetaclass, meta).__new__(meta, base_class.__name__+'Original', (base_class,), {'Meta': original_meta, '__module__': base_class.__module__})
@@ -136,14 +145,6 @@ class TypedModelMetaclass(ModelBase):
         if base_class:
             opts = cls._meta
 
-            # Dr Hacky "the hack" McHackerson, Sr.
-            # Django's deserialization code doesn't trigger sequence resets for proxy models.
-            # That's cool usually, but our deserialized models are alway going to be recasted
-            # to proxies, so the sequences will always have the wrong value after fixture loading.
-            # Turns out Django triggers sequence resets based on whether there's PK fields in
-            # opts.local_fields (which is normally empty for proxies.) So we hack it!
-            opts.local_fields[:] = base_class._meta.local_fields[:]
-
             # model_name was introduced in commit ec469ad in Django.
             if hasattr(opts, 'model_name'):
                 model_name = opts.model_name
@@ -204,7 +205,7 @@ class TypedModelMetaclass(ModelBase):
                 del cls._meta.__dict__['fields']
 
             def _fill_m2m_cache(self):
-                cache = SortedDict()
+                cache = OrderedDict()
                 for parent in self.parents:
                     for field, model in parent._meta.get_m2m_with_model():
                         if field in base_class._meta.original._meta.many_to_many or any(field in ancestor._meta.declared_fields.values() for ancestor in cls.mro() if issubclass(ancestor, base_class) and not ancestor==base_class):
@@ -258,6 +259,15 @@ class TypedModelMetaclass(ModelBase):
         return cls
 
 
+def get_deferred_class_for_instance(instance, desired_class):
+    """
+    Returns a deferred class (as used by instances in a .defer() queryset).
+    """
+    original_cls = instance.__class__
+    attrs = [k for (k, v) in original_cls.__dict__.items() if isinstance(v, DeferredAttribute)]
+    return deferred_class_factory(desired_class, attrs)
+
+
 class TypedModel(with_metaclass(TypedModelMetaclass, models.Model)):
     '''
     This class contains the functionality required to auto-downcast a model based
@@ -356,7 +366,12 @@ class TypedModel(with_metaclass(TypedModelMetaclass, models.Model)):
 
         self.type = typ
 
-        if self.__class__ != correct_cls:
+        current_cls = self.__class__
+
+        if current_cls != correct_cls:
+            if self._deferred:
+                # create a new deferred class based on correct_cls instead of current_cls
+                correct_cls = get_deferred_class_for_instance(self, correct_cls)
             self.__class__ = correct_cls
 
     def save(self, *args, **kwargs):
